<?php
/**
* Representation of groups as model, handles transactions for
* broups so they can be listed or queried. 
* 
* @author Smita Deshpande
* @package sourcemap
* @subpackage models
*/



class GroupsModel extends Model {

	/**
	 * Checks if the group exists
	 * @param string $groupname
	 * @return count (result)
	 */
	function checkGroupName($groupname) {	
		$this->db->select('*');				
		$this->db->from('groups');		
		$this->db->where('name', $groupname);

		$query = $this->db->get();

		return (count($query->result()) == 0); 
	}


	/**
	* Inserts the group information into 
	* the groups table
	*/
	function enterGroup($groupname, $description) {
		$data = array (
			'name' => addslashes($groupname),
			'slug' => url_title(strtolower($groupname)),
			'description'=> addslashes($description)
		);
		$this->db->insert("groups", $data);
		return $this->db->insert_id();
	}	

	/**
	* Enters the information into members table
	* with permissions as member, usernames
	*/
	function createMember($member, $group_id) {
		$id = $this->get_user_id($member);
		if($id != null) {
			$data = array ('user_id'  => $id,			
			   			   'group_id' => addslashes($group_id),
						   'username'  => addslashes(trim($member)),
						   'permissions' => "member");

			$this->db->insert('members', $data); 
		}
	}

	/**
	* All the admins are entered into the members table
	* @param interger $group_id, $admin as permissions
	*/
	function createAdmin($admin, $group_id) {
		$id = $this->get_user_id($admin);
		if($id != null) {
			$data = array ('user_id'  => $id,	
						   'group_id' => addslashes($group_id),		
						   'username'  => addslashes(trim($admin)),
						   'permissions' => "admin");		 
			$this->db->insert('members', $data); 
		}
	}

	/**
	* Gets the group name with the group id
	* @param integer $id
	* @return name
	*/
	function get_group_name($id) {
		$this->db->select('name');
		$this->db->from('groups');
		$this->db->where('id', $id);
		$query = $this->db->get();
		return array_pop($query->result())->name;
	}

	/**
	* Gets the group id from groups table
	* @param $name
	* @return integer id
	*/
	function get_group_id($name) {
		$this->db->select('id');
		$this->db->from('groups');
		$this->db->where('slug', $name);
		$query = $this->db->get();
		return array_pop($query->result())->id;
	}

	/**
	* Gets the entire row of the groups table
	* @param name
	* @return entire row
	*/
	function getGroupByName($name) {
		$this->db->select('*');
		$this->db->from('groups');
		$this->db->where('name', $name);
		$query = $this->db->get();
		return array_pop($query->result());
	}

	
	/*
	 * Get the entire row from the groups table
	 * @param string slug 
	 * @return entire row 
	 */
	function getGroupBySlug($slug) {
		$this->db->select('*');
		$this->db->from('groups');
		$this->db->where('slug', $slug);
		$query = $this->db->get();
		return array_pop($query->result());
	}
	

	/*
	 * Gets the slug from the name of the group
	 * @param string name
	 */

	function getSlugByName($name) {
	  $this->db->select('slug');
	  $this->db->from('groups');
	  $this->db->where('name', $name);
	  $query = $this->db->get();
	  return array_pop($query->result())->slug;

	}


	/**
	 * Get the group information with group id
	 * @param int $id
	 * @return entire 
	 */
	function getGroupByID($id) {
		$this->db->select('*');
		$this->db->from('groups');
		$this->db->where('id', $id);
		$query = $this->db->get();
		return array_pop($query->result());
	}

	/**
	 * Checks if the permissions is admin
	 * @param int $user_id, $group_id
	 * @return entire row
	 */
	function checkIsAdmin($user_id, $group_id) {
		$this->db->select('*');
		$this->db->from('members');
		$this->db->where('user_id', $user_id);
		$this->db->where('group_id', $group_id);
		$this->db->where('permissions', "admin");
		$query = $this->db->get();
		return (count($query->result()) >= 1);		
	}
	
	/**
	* Get the user id of the given member
	* @param string $member
	* @return $id if set otherwise null
	*/
	function get_user_id($member) {
		$this->db->select('id');
		$this->db->from('users');
		$this->db->where('name', $member);
		$query = $this->db->get();
		$id = array_pop($query->result());
		if(isset($id)) { return $id->id; } 
		else { return null; }
	}
	/**
	 * Gets the group information and the member info
	 * @param int $id
	 * @return entire row from groups table
	 */

	function getUsersGroups($id) {
		$this->db->select('*');		
		$this->db->from('groups');
		$this->db->where('user_id', $id);
		$this->db->join("members", "members.group_id = groups.id", "inner");
		$query = $this->db->get();
		
		return $query->result();		
	}

	
	
	/**
	* returns the count of the number of groups registered
	*/
	function getGroupsCount() { return $this->db->count_all_results('groups'); }

	/**
	* Gets the list of groups 
	* @limit limit number of groups
	* @sort group id in asc 
	* @offset null by default
	* @return array $grouplist
	*/
	function getGroups($limit = null, $offset =null, $sort = 'id ASC')
	{
		$this->db->select('*');
		$this->db->from('groups');

		$this->db->order_by($sort);
		$this->db->limit($limit, $offset);

		$query = $this->db->get();
		$grouplist = $query->result();

		return $grouplist;
	}

	/**
	* Gets the list of users 
	* @param integer $group_id
	* @return array $user_list
	*/
	function get_users($group_id) {
		$this->db->select('username');
		$this->db->from('members');
		$this->db->where('group_id', $group_id);
		$query = $this->db->get();
		$members = $query->result();

		$usernames = array();
		foreach($members as $member) { $usernames[] = $member->username; }	  

		if(count($usernames) == 0) { return null; }
		$this->db->select('*');
		$this->db->from('users');
		$this->db->where_in('name', $usernames);
		$this->db->join("members", "members.user_id = users.id", "inner");
		$this->db->where("members.group_id", $group_id);
		$query = $this->db->get();
		$user_list = $query->result_array();

		for($i=0; $i < count($user_list); $i++) {
			$this->db->select('*');
			$this->db->where('creator', $user_list[$i]['id']);
			$this->db->from('objects_complex');	    $count = $this->db->count_all_results();
			$user_list[$i]['smapcount'] = $count;
		} 
		return $user_list;
	}
	
	/**
	* Returns the top 5 groups based on the number of group memebers it has
	*/
	function get_top_groups() {
		$this->db->select('group_id');
		$this->db->from('members');
		$this->db->group_by('group_id');
		$this->db->order_by('count(*) desc');
		$this->db->limit(5);
		$query = $this->db->get();
		$group = $query->result_array();
		$top_groups= array();

		foreach($group as $k => $v) {
			$this->db->select('name');
			$this->db->from('groups');
			$this->db->where('id', $v["group_id"]);
			$result = $this->db->get();
			$top_groups[] = $result->result_array();
		}
		return $top_groups;
	}
}