<?php

/**
 * Representation of parts as model, handles transactions for
 * raw (non object) parts so they can be listed or edited. 
 * 
 * @author Matthew Hockenberry
 * @package sourcemap
 * @subpackage models
 */

class PartsModel extends Model{
	
	/**
	 * @ignore
	 */
	function PartsModel(){
		parent::Model();	
	}	
	
	/**
	 * Gets a given raw parts by id.
	 * @param integer $mid The part id 
	 * @return Object Object containing parts properties or false if none selected.
	 */
	function getPartByID($mid){
			$this->db->select('*');		
			$this->db->from('parts_list');
			$this->db->where('id', $mid);

			$query = $this->db->get();
			return array_pop($query->result());
	}
	
	/**
	 * Gets a given raw parts by name. DEPRECIATED
	 * @param string $name The part name 
	 * @return Object Object containing parts properties or false if none selected.
	 */
	function getPartByName($name){
			$this->db->select('*');		
			$this->db->from('parts_list');
			$this->db->where('name', $name);

			$query = $this->db->get();
			return (array_pop($query->result()));
	}
	

	/**
	 * Returns the object from the parts_list
	 * @param string $slug
	 */
	function getPartBySlug($slug){
			$this->db->select('*');		
			$this->db->from('parts_list');
			$this->db->where('slug', $slug);

			$query = $this->db->get();
			return (array_pop($query->result()));
	}
	/**
	 * Gets the list of all raw parts.
	 * @return Array Array of all raw parts.
	 * @param integer $limit Limit number of parts.
	 * @param integer $offset Offset to return parts.
	 * @param string $sort How parts should be sorted (default class ASC, name ASC).
	 */
	function getParts($limit = null, $offset = null, $sort = 'class ASC, name ASC'){
		$this->db->select('*');		
		$this->db->from('parts_list');
		$this->db->where('approval_status', 'approved');
		
		$this->db->order_by($sort);		
		
		$this->db->limit($limit, $offset);
		
		$query = $this->db->get();
		return  $query->result();
	}


	/**
	 * Returns the objects from source_locations
	 * @param int $id
	 */
	function getPartLocations($id){
		$this->db->select('*');		
		$this->db->from('source_locations');
		$this->db->where('mid', $id);
		
				
		$query = $this->db->get();
		return  $query->result();
	}
	/**
	 * Gets the list of all raw parts requiring approval.
	 * @return Array Array of all raw parts requiring approval.
	 * @param integer $limit Limit number of parts.
	 * @param integer $offset Offset to return parts.
	 * @param string $sort How parts should be sorted (default class ASC, name ASC).
	 */
	function getPartsForApproval($limit = null, $offset = null, $sort = 'class ASC, name ASC'){
		$this->db->select('*');		
		$this->db->from('parts_list');
		$this->db->order_by($sort);		
		$this->db->where('approval_status', 'notapproved');
		
		$this->db->limit($limit, $offset);
		
		$query = $this->db->get();
		return $query->result();
	}
	
	/**
	 * Gets a given raw part name.
	 * @param int $mid The part id.
	 * @return string Name of part.
	 */
	function getPartName($mid){
		$this->db->select('name');		
		$this->db->from('parts_list');
		$this->db->where('id', $mid);

		$query = $this->db->get();
		$elem = array_pop($query->result());
		
		return $elem->name;
	}
	/**
	 * Gets the class of a raw part.
	 * @param int $mid The part id.
	 * @return string Class of part.
	 */
	function getPartClass($mid){
		$this->db->select('class');		
		$this->db->from('parts_list');
		$this->db->where('id', $mid);

		$query = $this->db->get();
		$elem = array_pop($query->result());
		
		return $elem->class;	
	}
	/**
	 * Gets a given raw part CO2.
	 * @param int $mid The part id.
	 * @return float CO2 of part.
	 */
	function getPartCO2($mid){
		$this->db->select('co2_eq');		
		$this->db->from('parts_list');
		$this->db->where('id', $mid);

		$query = $this->db->get();
		$elem = array_pop($query->result());
		
		return $elem->co2_eq;
	}
	/**
	 * Gets a given raw part impact.
	 * @param int $mid The part id.
	 * @return float Impact of part.
	 */
	function getPartImpact($mid){
		$this->db->select('impact');		
		$this->db->from('parts_list');
		$this->db->where('id', $mid);

		$query = $this->db->get();
		$elem = array_pop($query->result());
		
		return $elem->impact;
	}
	/**
	 * Gets a given raw part description.
	 * @param int $mid The part id.
	 * @return string Description of part.
	 */
	function getPartDescription($mid){
		$this->db->select('description');		
		$this->db->from('parts_list');
		$this->db->where('id', $mid);

		$query = $this->db->get();
		$elem = array_pop($query->result());
		
		return $elem->description;	
	}
	
	/**
	 * Returns the total size of the raw part table.
	 * @return int Number of raw parts.
	 */
	function getPartCount(){
		return $this->db->count_all_results('parts_list');
	}
	/**
	 * @todo Should save the user who suggested it as a point of contact
	 */
	function savePart($part) {
		$data = array(
			'name' => addslashes($part->name),
			'class' => $part->classification,
			'co2_eq' => $part->emissions,			
			'unit' => addslashes($part->unit),
			'description' => addslashes($part->description),
			'uid' => "1",
			'impact' => "0",
			'approval_status' => "notapproved"						
		);

		$this->db->insert('parts_list', $data);
	}


	/**
	 * Updates the parts_list table with the new approval_status
	 * @param int $partID
	 */
	function approvePart($partID) {
		$data = array(
			'approval_status' => "approved"			
		);
		$this->db->where('id', $partID);
		$this->db->update('parts_list', $data);
	}
	

	/**
	 * Gets the count of all the results in parts_list
	 * @param string $searchTerm
	 */
	function getPartSearchCount($searchTerm){
		$this->db->or_like('name', $searchTerm);
		$this->db->or_like('description', $searchTerm);
		return $this->db->count_all_results('parts_list');
	}
	

	/**
	 * Returns the results from the parts_list
	 * @orderby class and name in ascending order
	 * @limit int $limit
	 */
	function searchParts($searchTerm, $limit = null, $offset = null, $sort = 'class ASC, name ASC'){
		$this->db->select('*');		
		$this->db->from('parts_list');
		$this->db->where('approval_status', 'approved');
		$this->db->like('name', $searchTerm);
		$this->db->or_like('description', $searchTerm);
		$this->db->or_like('class', $searchTerm);
		
		$this->db->order_by($sort);		
		
		$this->db->limit($limit, $offset);
		
		$query = $this->db->get();
		return  $query->result();
	}

	/**
	 * Returns the result from tags table
	 * @param int $mid
	 */
	function getTagsForPart($mid){
		$this->db->select('*');
		$this->db->from('tags');
		$this->db->where('id', $mid);
		$this->db->where('type', 'part');
		
		$query = $this->db->get();
		return  $query->result();
	}
	
	
	/**
	 * Inserts the new data into tags table
	 * @param string $text, int $mid
	 */
	function addTagsForPart($text, $mid) {
		$data = array(
			'type' => 'part',
			'name' => $text,
			'id' => $mid
		);
	
		$this->db->insert('tags', $data);
	}
}
?>