/**
 * An OpenLayers based map visualization.
 *
 * @version 0.8
 * @author sourcemap@media.mit.edu
 * @package sourcemap
 * @subpackage js
 * @class
 * @param {String} mapDiv The ID of the map div to contain the map.
 * @param {Sourcemap.Object} config The object and all parts.
 * @param {Object} options A dictionary of options.
 */
Sourcemap.OpenSourcemap = function( mapDiv, config, options ) {
	this.map = null;
	this.features = null;
	this.selectControl = null;
	this.selectedMarker = null;
	this.objectsummary = config;
	this.options = options || {};
	this.printmap = !!this.options.printmap;
	this.showUserLocation = !!this.options.userLocation;
	this.hideDescriptions = !!this.options.hideDescriptions;
	this.partMarker = [];
	this.partLines = [];
	this.partLocations = [];

	// used for monitoring the position of objects on the map
	this.lastCenter = null;
	this.loaded = false;
	
	//Lines and parts to be expanded from a part in this object
	this.subParts = [];
	this.subLines = [];

	this.deliveryLine = null;
	this.destinationMarker = null;

	this.originMarker = null;
	this.userMarker = null;

	this.origin = null;
	this.destination = null;
	this.userLocation = this.options.userLocation || {};
	this.popup = null;
	
	this.lastHash = null;
	this.currentlyExpanded = 0;
	
	var lightGreen = '008000';
  
	/**
	 * Create an OpenLayers map on the page.
	 * @param {String} mapName The id of the html element that will recieve the
	 * OpenLayers slippy map.
	 */
	this.createMap = function (mapName) {
		var mapOptions = {
			projection: new OpenLayers.Projection( "EPSG:900913" ),
			displayProjection: new OpenLayers.Projection( "EPSG:4326" ),
			maxExtent: new OpenLayers.Bounds(-20037508.34, -20037508.34,
						                             20037508.34, 20037508.34),
			theme:Sourcemap.baseurl + 'assets/scripts/OpenLayers-2.8/theme/default/style.css'
		};

		var nav = new OpenLayers.Control.Navigation();
		
		// keep this clicked point stationary, but increase the zoom
		nav.defaultDblClick = function(evt) {
			var clickPt = this.map.getLonLatFromViewPortPx( evt.xy );
			var centerPt = this.map.getCenter();
			
			centerPt.lon = (clickPt.lon + centerPt.lon)/2.0;
			centerPt.lat = (clickPt.lat + centerPt.lat)/2.0;			
			this.map.setCenter( centerPt, this.map.zoom + 1 );
		};
		
		mapOptions.controls = [ 
			nav
		];
	
		if (typeof(this.options.navPosition) != 'undefined') {
			if (this.options.navPosition != null) {
				mapOptions.controls.push( new OpenLayers.Control.PanZoom({ 
					position: this.options.navPosition 
				}) );
			}
		}
		else {
			mapOptions.controls.push( new OpenLayers.Control.PanZoom() );
		}
		
		if(Sourcemap.tileprovider == "googlemaps") { 			
			var glayer = new OpenLayers.Layer.Google( "Base Map", {
				sphericalMercator:true,
				type: G_PHYSICAL_MAP,
				wrapDateLine: true
			} );
		}
		else if(Sourcemap.tileprovider == "cloudmade"){
			if(this.printmap) { style = 11015;} else { style = 4993; }
			var glayer = new OpenLayers.Layer.CloudMade("CloudMade", {
			    key: '3bb82639a274433fb30120279ea08c76',
			    styleId: style,
				sphericalMercator:true,
				wrapDateLine: true			
			});
			// BC9A493B41014CAABB98F0471D759707 
		}
		this.features = new OpenLayers.Layer.Vector( "Objects & Parts", {
			projection: mapOptions.projection,
			displayProjection: mapOptions.displayProjection,
			wrapDateLine: true
		});
	
		OpenLayers.ImgPath = Sourcemap.baseurl+"assets/scripts/OpenLayers-2.8/img/";
		this.map = new OpenLayers.Map( mapName, mapOptions );
	
		this.selectControl = new OpenLayers.Control.SelectFeature( this.features,
			{ onSelect: this.markerSelect, onUnselect: this.markerUnselect, scope: this } );
		this.map.addControl( this.selectControl );
	
		this.selectControl.activate();
	
		this.map.addLayers( [glayer, this.features] );
		
		this.map.events.register( 'moveend', this, this.mapMoved );

		this.map.setCenter( new OpenLayers.LonLat(0,0), 1 );
		
		//Allows "back" and "forward" buttons to expand previous/next
		//part's subparts. Small performance penalty, though, since this is called once
		// per second. "that" and "this" are a hack to make sure we're in the right 
		//scope when calling expandObjectFromURL. JS is annoying that way.
		setInterval(function(that) { that.expandObjectFromURL(); }, 1000, this );
	};

	/**
	 * Load an object's data into this OpenSourcemap.
	 * @param {Sourcemap.Object} summary An object summary, parts and all.
	 */  
	this.loadMap = function (summary) {
		if(summary.origin)
			this.setOrigin(summary);
		if(summary.destination)
			this.setDestination(summary);
		if(this.showUserLocation && this.userLocation) {
			this.setUserLocation(this.userLocation);
		}
	
		for(var i = 0; i < summary.parts.length; i++ ){
			if ( summary.parts[i].flag != 'remove' ) {
				this.setPart(summary.parts[i],i+1);
			}
		}

		var allFeatures = this.partLines;

		allFeatures = allFeatures.concat( this.partMarker );

		if ( this.destinationMarker ) {
			allFeatures.push( this.deliveryLine );
			allFeatures.push( this.destinationMarker );
		}

		if ( this.originMarker ) {
			allFeatures.push( this.originMarker );
		}

		if ( this.userMarker ) {
			allFeatures.push( this.userMarker );
		}

		this.features.addFeatures( allFeatures );

		if(summary.origin)
			this.fitMap();

		this.lastCenter = this.map.getCenter();
		this.lastCenter = { ll:this.lastCenter, px:this.map.getLayerPxFromLonLat(this.lastCenter) };
		this.loaded = true;
	};
  
	/**
	 * Refresh the map.  This method removes all features from the map,
	 * then reloads the map based on the initial object configuration.
	 */
	this.refreshMap = function() {
		// this flag is reset after the loadMap call at the end of this method.
		this.loaded = false;

		this.features.removeFeatures( this.features.features );
				
		this.partLines = [];
		this.partMarker = [];
		this.partLocations = [];

		this.loadMap(this.objectsummary);
	};

	/**
	 * Set the map extent to the zoom/position that shows all features.
	 */
	this.fitMap = function (constrain) {
		if(this.originMarker == null)
			return false;

		var lines = this.partLines;

		if ( this.deliveryLine != null )
			lines.push( this.deliveryLine );

		var bounds = new OpenLayers.Bounds();

		// get the origin (but only 1 origin -- the one that is within real projection bounds)
		var theOrig = this.originMarker.geometry.getVertices();
		for (var i=0; i< theOrig.length; i++) {
			if ( theOrig[i].x > - CIRCUM && theOrig[i].x < CIRCUM ) {
				theOrig = theOrig[i];
				break;
			}
		}

		if(lines.length == 0) {
			this.map.zoomTo(16);
		}
		else {
			// for each line emanating from the origin
			for (var i=0; i< lines.length; i++) {
				if ( lines[i] ) {
					if ( lines[i].geometry ) {
						// components in a geometry collection
						if ( 'components' in lines[i].geometry ) {
							this.extendRealBounds( theOrig, lines[i].geometry, bounds );          
						}
						// a simple geometry
						else {
							bounds.extend(lines[i].geometry.getBounds());
						}
					}
				}
			}

			if ( this.options.regional ) {
				var w = bounds.getWidth();
				var h = bounds.getHeight();

				bounds = new OpenLayers.Bounds( theOrig.x - w/8, theOrig.y - h/8, 
					theOrig.x + w/8, theOrig.y + h/8);
			}
			
			this.map.zoomToExtent( bounds );
			// I would think that these would effect the zoom level of the map, but no.
			//var resolution = this.map.getResolutionForZoom( this.map.getZoom() );
			//this.map.setOptions({ maxExtent: bounds, minResolution: resolution });

			var constrain = ( typeof(constrain) != 'undefined' ) ? constrain : this.options.constrain;
			if ( constrain ) {
				var pxl = bounds.getCenterPixel();
				var bll = bounds.getCenterLonLat();
				var mw = this.map.div.clientWidth;
				var constr = this.options.constrain.split(/_/);
				var px = parseInt(constr[0],10);
				var pos = constr[1];
				
				var w2 = mw - px;
				var x2 = (w2/2) + this.map.div.clientLeft;
								
				var newLL = this.map.getLonLatFromPixel( new OpenLayers.Pixel( x2, pxl.y ) );
				var oldLL = this.map.getCenter();
				var dlon = newLL.lon - oldLL.lon;

				this.map.setCenter( new OpenLayers.LonLat( bll.lon - dlon, bll.lat ) );
			}
		}
	};
  
	/**
	 * Set the origin of the map.
	 * @param {Sourcemap.Object} summary The object summary.
	 */
	this.setOrigin = function (summary){
		if ( summary.latlon && summary.latlon != "0|0") {
			var coords = summary.latlon.split(/\|/);

			this.origin = new OpenLayers.LonLat( coords[1], coords[0] )
				.transform( this.map.displayProjection, this.map.projection );
			if (this.originMarker != null) {
				this.features.removeFeatures([this.originMarker]);
			}

			var originDescription = this.hideDescriptions ? '' : summary.description;
			this.originMarker = this.makeMarker(this.origin, '+', '004000', "<strong>"+Sourcemap.Util.utf8decode(summary.name) + '</strong> from ' + Sourcemap.Util.utf8decode(summary.origin) + '<br/>' + Sourcemap.Util.utf8decode(originDescription));

			if(this.originMarker != null && this.destinationMarker != null){
				this.deliveryLine = this.dsAddLine(this.origin, this.destination, "000000");
			}

			for(var k = 0; k < this.partLocations.length; k++){
				this.partLines[k] = this.dsAddLine(this.origin, this.partLocations[k], "004000");
			}
		}	
	};

	/**
	 * Remove the origin marker from the map.
	 */
	this.removeOrigin = function () {
		this.features.removeFeatures([this.originMarker]);
		this.originMarker = null;
	};
  
	/**
	 * Remove a part from the map.
	 * @param {Sourcemap.Part} part The part object with origin property.
	 */
	this.removePart = function (part) {	
		this.features.removeFeatures( [this.partMarker[part.origin]] );
	};

	/**
	 * Set the user location on the map.
	 * @param {Object} location A config object with latitude, longitude, and 
	 * placename properties.
	 */  
	this.setUserLocation = function( location ) {

		// get the position of the user, and transform it to the map projection
		var point = new OpenLayers.LonLat(location.longitude, location.latitude)
			.transform( this.map.displayProjection, this.map.projection );

		if (this.userMarker != null) {
			this.features.removeFeatures([this.userMarker]);
		}

		// make a new marker
		this.userMarker = this.makeMarker( point, '-', '400000', "Delivered to you in " + location.placename );

		if ( this.destinationMarker != null ) {
			// draw line from destination to user
			this.partLines.push( this.dsAddLine( this.destination, point, '400000' ) );
		}
		else if ( this.originMarker != null ) {
			// draw line from origin to user if no destination
			this.partLines.push( this.dsAddLine( this.origin, point, '400000' ) );
		}
	};
	
	/**
	 * Remove the user location from the map.
	 */
	this.removeUserLocation = function() {
		if ( this.userMarker != null ) {
			this.features.removeFeatures( [this.userMarker] );
			this.userMarker = null;
		}
	};
  
	/**
	 * A utility function to extend bounds to contain a geometry
	 * specifically, a multi-part geometry.
	 * @param {OpenLayers.Geometry} theOrig A point.
	 * @param {OpenLayers.GeometryCollection} geometry A collection of geometries.
	 * @param {OpenLayers.Bounds} bounds The bounds that get extended.
	 */
	this.extendRealBounds = function( theOrig, geometry, bounds ) {
		// for each line in the multipart geometry
		for (var j=0; j< geometry.components.length; j++) {
			// get the component and it's endpoints
			var cmp = geometry.components[j];
			var verts = cmp.getVertices(true);

			// for each of these endpoints
			for (var k=0; k< verts.length; k++) {
				// if one of the coordinates is the origin
				if ( verts[k].x == theOrig.x && verts[k].y == theOrig.y ) {
					bounds.extend( cmp.getBounds() );
					return;
				}
			}
		}
	};

	/**
	 * Set the destination of the map.
	 * @param {Sourcemap.Object} summary An object summary, parts and all.
	 */
	this.setDestination = function (summary) {
		// if the origin lat and lon are set
		if( summary.latlon && summary.latlon != "0|0") {
			// get the position of the destination, and transform it to the map projection
			var coords = summary.latlon.split(/\|/);
			this.destination = new OpenLayers.LonLat( coords[1], coords[0] )
				.transform( this.map.displayProjection, this.map.projection );
	
			// remove the marker if it exists
			if (this.destinationMarker != null)
				this.features.removeFeatures([this.destinationMarker]);
	
			// make a new marker
			this.destinationMarker = this.makeMarker(this.destination, '-', '000000', 'Delivered to: ' + Sourcemap.Util.utf8decode(summary.destination) );
	
			// draw a line from origin to destination
			if (this.originMarker != null && this.destinationMarker != null) {
				this.deliveryLine = this.dsAddLine(this.origin, this.destination, "000000");
			}
		}
	};

	/**
	 * Set a part on the map.
	 * @param {Sourcemap.Part} part The part to add to the map.
	 * @param {Number} num The index of the part.
	 */
	this.setPart = function(part,num){
		// if the origin lat and lon are set
		if(part.latlon && part.latlon != "0|0") {
			// get the position of the part, and transform it to the map projection
			var coords = part.latlon.split(/\|/);
			var point = new OpenLayers.LonLat( coords[1], coords[0] )
				.transform( this.map.displayProjection, this.map.projection );
	
			this.partLocations[num-1] = point;
	
			// remove the marker if it exists
			// num is the count, we want the index(-1) of the previous item(-1)
			var partInfo = null;
			for ( var i = 0; i < this.partMarker.length; i++ ) {
				if ( this.partMarker[i].geometry.intersects( 
					new OpenLayers.Geometry.Point( point.lon, point.lat ) ) ) {
					partInfo = this.partMarker[i].data;
					//Sourcemap.debug( 'point intersects geometry: ' + point.lon + ',' + point.lat );

					this.features.removeFeatures( [this.partMarker[i]] );
					this.partMarker.splice( i, 1 );
					//Sourcemap.debug( 'removed existing marker from marker list' );
					break;
				}
			}
	
			// make a new marker
			if ( partInfo ) {
				// append this info to the current marker
				var ps = partInfo.info.match( /<p/g );
				if ( ps.length == 1 ) {
					partInfo = partInfo.info.substr( 0, 31 ) +
						"<img src=\"" + this.makeMarkerImage( "008000", partInfo.label, "ffffff", 16 ) + "\"/>" +
						partInfo.info.substr( 31 );
				}
				else {
					partInfo = partInfo.info;
				}
				
				if(typeof(this.objectsummary.parts[num-1].linkid) != 'undefined' && this.objectsummary.parts[num-1].linkid != 0) {
					var partLink = Sourcemap.baseurl + Sourcemap.Util.utf8decode(this.objectsummary.parts[num-1].linkid);
					var partName = "<a href='"+partLink+"'><strong>" + Sourcemap.Util.utf8decode(this.objectsummary.parts[num-1].name) + "</strong></a> ";
				}
				else { var partName = "<strong>" + Sourcemap.Util.utf8decode(this.objectsummary.parts[num-1].name) + "</strong> ";}
				
				var partDescription = this.hideDescriptions ? '' : this.objectsummary.parts[num-1].description;
				if(partDescription == "") { descriptionbreak = ""; } else { descriptionbreak = "<br/>";}
				var feature =  this.makeMarker(point, 
					'...', 
					'008000', 
					partInfo +
					"<p class=\"popupMultiInfo\"><img src=\"" + this.makeMarkerImage( "008000", num, "ffffff", 16 ) + "\"/>" +
					partName + "<br/>" +
					partDescription + descriptionbreak + "<strong>Footprint:</strong> "+this.objectsummary.parts[num-1].emissions+"kg CO2e<br/>" 
					+ "</p>" 
				);
				//Add extra information to the marker; used to expand parts
				if(typeof(this.objectsummary.parts[num-1].linkid) != 'undefined') {
					feature.linkid = Sourcemap.Util.utf8decode(this.objectsummary.parts[num-1].linkid);
				} else {
					feature.linkid = 0;
				}
				feature.number = num;
				this.partMarker.push(feature);
			}
			else {
				// add a new marker
				if(typeof(this.objectsummary.parts[num-1].linkid) != 'undefined' && this.objectsummary.parts[num-1].linkid != 0) {
					var partLink = Sourcemap.baseurl + Sourcemap.Util.utf8decode(this.objectsummary.parts[num-1].linkid);
					var partName = "<a href='"+partLink+"'><strong>" + Sourcemap.Util.utf8decode(this.objectsummary.parts[num-1].name) + "</strong></a> ";
				}
				else { var partName = "<strong>" + Sourcemap.Util.utf8decode(this.objectsummary.parts[num-1].name) + "</strong> ";}

				var partDescription = this.hideDescriptions ? '' : this.objectsummary.parts[num-1].description;
				var feature = this.makeMarker(point, 
					num, 
					'008000', 
					"<p class=\"firstPopupMultiInfo\">"+ partName +
					"from " + Sourcemap.Util.utf8decode(this.objectsummary.parts[num-1].origin) + "<br/><strong>Footprint:</strong> "+this.objectsummary.parts[num-1].emissions+"kg CO2e<br/>" +
					partDescription + "</p>"
				);
				//Add extra information to the marker; used to expand parts
				if(typeof(this.objectsummary.parts[num-1].linkid) != 'undefined') {
					feature.linkid = this.objectsummary.parts[num-1].linkid;
				} else {
					feature.linkid = 0;
				}
				feature.number = num;
				this.partMarker.push(feature);
			}
	
			// draw line from origin to assembly
			if (this.originMarker != null && this.partLocations[num-1] != null) {
				this.partLines.push( this.dsAddLine(this.origin, point, "004000") );
			}
		}
	};
  
	/**
	 * Make a marker for the map.
	 * @param {OpenLayers.LonLat} point The location of the marker.
	 * @param {String} label The label of the marker.
	 * @param {String} prcolor The color of the marker.
	 * @param {String} info Extra object info to display in the popup.
	 * @return {OpenLayers.Feature.Vector} A feature at the specific point.
	 */
	this.makeMarker = function(point, label, prcolor, info){
		var iconOptions = {
			label: label
		};

		iconOptions.primaryColor = prcolor;
		iconOptions.labelColor = "ffffff";
		
		if(this.printmap) { iconOptions.primaryColor = "000000"; iconOptions.size = 16;}
		else { iconOptions.size = 32; }
		// get a google chart for the map marker (same technique as MapIconMaker)
		var markerUrl = this.makeMarkerImage( iconOptions.primaryColor, iconOptions.label, iconOptions.labelColor, iconOptions.size );

		var marker = this.dsAddPoint(
			/* geometry */ new OpenLayers.Geometry.Point( point.lon, point.lat ), 
			/* attributes */ { label: label, info: info },
			/* style */ {
				externalGraphic: markerUrl,  
				graphicWidth: iconOptions.size,
				graphicHeight: iconOptions.size,
				graphicXOffset: -(iconOptions.size/2),
				graphicYOffset: -(iconOptions.size/2)
			}
		);

		return marker;
	};
  
	/**
	 * Add a point to the map in a Dateline Safe fashion. Dateline safe means the
	 * feature layer will replicate the specified features on either side of the 
	 * globe to prevent disappearing map markers if the users scrolls over the date
	 * line.
	 * @param {OpenLayers.Geometry.Point} geometry The point location.
	 * @param {Object} attributes A dictionary of extra point attributes.
	 * @param {Object} style An OpenLayers style definition.
	 * @return {OpenLayers.Feature.Vector}
	 */
	this.dsAddPoint = function( geometry, attributes, style ) {
		var mpt = null;

		if (this.originMarker) {
			var dist = this.originMarker.geometry.distanceTo(geometry);
			if ( dist > CIRCUM/2) {
				if ( this.originMarker.geometry.x > geometry.x ) 
					geometry.x += CIRCUM;
				else // if ( this.originMarker.geometry.x < geometry.x ) 
					geometry.x -= CIRCUM;
			}
		}

		return new OpenLayers.Feature.Vector( geometry, attributes, style );
	};

	/**
	 * Create an URL for a marker image. This method depends on the google chart
	 * api to generate a small, static image for a map marker.
	 * @param {String} primaryColor The primary, or background, color.
	 * @param {String} label The label inside of the marker.
	 * @param {String} labelColor The color of the label text.
	 * @param {Number} size The size of the marker.
	 * @return {String} A marker image URL.
	 */  
	this.makeMarkerImage = function( primaryColor, label, labelColor, size ) {
		return 'http://chart.apis.google.com/chart?cht=it&chs=' + size + 'x' + size + '&chco=' + primaryColor + 
			',000000ff,ffffff01&chl=' + encodeURIComponent(label) + '&chx=' + labelColor + 
			',' + (size/2).toFixed() + '&chf=bg,s,00000000&ext=.png';
	};

	/**
	 * Make a polyline between two points with a specified color. This polyline
	 * function creates arcs that slowly curve over the map, attenuating to no arc
	 * along equal longitude. 
	 * @param {OpenLayers.LonLat} from One endpoint of the line.
	 * @param {OpenLayers.LonLat} to The other endpoint of the line.
	 * @return {OpenLayers.Geometry.LineString} A polyline.
	 */
	this.makePolyline = function(from, to){
		// create a 'bent' linestring
		var res = 8.0; // resolution
		var pts = new Array();
		var dx = to.lon - from.lon;
		var dy = to.lat - from.lat;
		var theta = (Math.PI / 2) - Math.atan( dy/dx );
		var maxdisp = Math.sqrt(dx*dx+dy*dy) * 0.05;

		if ( dx == 0 && dy == 0 ) {
			pts.push( new OpenLayers.Geometry.Point( from.lon, from.lat ) );
		}
		else {
			for ( var p = 0; p < res; p++ ) {
				var relamt = Math.sin(p/res*Math.PI) * maxdisp;
				if ( Math.abs(Math.sin(theta)) < Math.abs(Math.cos(theta)) ) {
					relamt *= Math.abs(Math.sin(Math.PI*dx/dy));
				}
				var ddx = Math.cos(theta+Math.PI) * relamt;
				var ddy = Math.sin(theta) * relamt;

				pts.push( new OpenLayers.Geometry.Point( from.lon + (dx*p/res) + ddx, from.lat + (dy*p/res) + ddy ) );
			}
		}
		pts.push( new OpenLayers.Geometry.Point( to.lon, to.lat ) );
		return new OpenLayers.Geometry.LineString(pts);
	};

	/**
	 * Add a line to the map in a Dateline Safe fashion. Dateline safe means the
	 * feature layer will replicate the specified features on either side of the 
	 * globe to prevent disappearing lines if the users scrolls over the date
	 * line.
	 * @param {OpenLayers.Geometry.Point} from An endpoint location.
	 * @param {OpenLayers.Geometry.Point} to An endpoint location.
	 * @param {String} color The color of the line.
	 * @return {OpenLayers.Feature.Vector}
	 */
	this.dsAddLine = function( from, to, color ) {
		var lines = new OpenLayers.Geometry.MultiLineString();
		if(this.printmap) { color = '#000000'; width = 2;}
		else {color = ( color.indexOf('#') != 0 ) ? '#' + color : color; width = 5;}
		
		if ( Math.abs(to.lon - from.lon) > CIRCUM / 2 )
		{
			if ( to.lon > from.lon && from.lon + CIRCUM - to.lon < to.lon - from.lon ) {
				var fromp = new OpenLayers.LonLat( from.lon + CIRCUM, from.lat );
				lines.addComponents( [ this.makePolyline( fromp, to ) ] );
			}
			else {
				var top = new OpenLayers.LonLat( to.lon + CIRCUM, to.lat );
				lines.addComponents( [ this.makePolyline( from, top ) ] );
			}
		}
		else {
			lines.addComponents( [ this.makePolyline( from, to ) ] );
		}

		return new OpenLayers.Feature.Vector(
			/* geometry */ lines, 
			/* attributes */ {},
			/* style */ { 
				strokeWidth: width,
				strokeOpacity: 0.8,
				strokeColor: color
			}
		);
	};

	//Sends an ajax request for an part's subparts and expands them
	this.doPost = function(linkid, point, num, scope) {
		$.post(Sourcemap.baseurl+"objects/getjsonbylinkid", { linkid: linkid }, function(data) {scope.expandParts(data, point, num, scope);}, "json");
	};

	/**
	 * A callback to process the selection of a marker.
	 * @param {OpenLayers.Feature.Vector} feature The feature that has been selected.
	 */
	this.markerSelect = function( feature ) {
		if( feature.geometry instanceof OpenLayers.Geometry.LineString )
			return false;
		if(typeof(feature.linkid) != 'undefined' && feature.linkid != 0) {
			this.doPost(feature.linkid, feature.geometry.getBounds().getCenterLonLat(), feature.number, this);
		}
		this.selectedMarker = feature;
		$("body").trigger("partSelected", [feature.attributes.label]);
	
		if ( this.popup == null ) {
			this.popup = new OpenLayers.Popup.FramedCloud( 'Marker',
				feature.geometry.getBounds().getCenterLonLat(),
				null,
				'<div style="padding:8px;">' + feature.attributes.info + '</div>',
				null,
				true,
				this.selectControl.unselect( this.selectedMarker )
			);
		}
		else {
			this.popup.contentHTML = '<div style="padding:8px;">' + feature.attributes.info + '</div>';
			this.popup.lonlat = feature.geometry.getBounds().getCenterLonLat();
		}
			
		this.map.addPopup( this.popup );
		this.popup.show();
	
		var size1 = this.popup.contentDiv.firstChild.getBoundingClientRect();
		var size2 = this.popup.contentDiv.getBoundingClientRect();
	
		// if the first child of the popup content wants to be bigger than the available size
		if ( (size1.top - size1.bottom) != (size2.top - size2.bottom) ) {
			this.popup.contentDiv.style.overflowY = 'scroll';
		}
	};

	/**
	 * A callback to process the deselection of a map marker.
	 * @param {OpenLayers.Feature.Vector} feature The feature to deselect.
	 */
	this.markerUnselect = function( feature ) {
		this.clearPopup();
	};

	/**
	 * Clear the popups on the map. This is useful for eliminating map clutter
	 * without searching all features and un-selecting them.
	 */
	this.clearPopup = function() {
		if ( this.popup != null )
			this.map.removePopup( this.popup );
	};

	/**
	 * A callback to process the movement of the map. This method specifically
	 * addresses the size of the popup windows when the popup is first shown
	 * and the map pans slightly to show the full popup content.
	 * @param {Object} event The move event.
	 */
	this.mapMoved = function( event ) {
		if ( !this.loaded ) return;
		var theCenter = this.map.getCenter();
		theCenter = { ll: theCenter, px: this.map.getLayerPxFromLonLat( theCenter ), zoom: this.map.getZoom() };

		if ( this.lastCenter && this.lastCenter.zoom == theCenter.zoom )
		{
			// dlon / dpix should remain relatively constant
			// this should remain very near zero, if it is high (usually 80K or so),
			// that indicates that the dateline has been crossed
			var shiftAmt = this.map.getResolution() - (this.lastCenter.ll.lon - theCenter.ll.lon)/(this.lastCenter.px.x - theCenter.px.x);
			if ( shiftAmt > 80000 )
			{
				var numChanged = 0;
				var theMap = this.map;
				var shiftGeom = function(geometry,amt){
					if (geometry.components) {
						for (var i = 0; i < geometry.components.length; i++ ) {
							shiftGeom(geometry.components[i],amt);
						}
					}
					else {
						numChanged++;
						geometry.x += amt;
					}
					geometry.setBounds(theMap.getExtent());
				};
				var amt = (this.lastCenter.ll.lon < theCenter.ll.lon) ? CIRCUM : -CIRCUM;
				var flen = this.features.features.length;
				var fcopy = [];
				for ( var i = 0; i < flen; i++ )
				{
					fcopy.push( new OpenLayers.Feature.Vector(
						/* geometry */ this.features.features[i].geometry,
						/* attributes */ this.features.features[i].attributes,
						/* style */ this.features.features[i].style
					));
				}
				var ext1 = this.features.getDataExtent();
				this.features.removeFeatures(this.features.features);
				for ( var i = 0; i < flen; i++ )
				{
					shiftGeom(fcopy[i].geometry, amt);
				}
				this.features.addFeatures(fcopy);
				var ext2 = this.features.getDataExtent();
				this.features.redraw();
			}
			//Sourcemap.debug('Shifted value: '+shiftAmt);
		}

		if ( this.selectedMarker != null && this.selectedMarker.popup != null ) {
			var popup = this.selectedMarker.popup;
			
			var size1 = popup.contentDiv.firstChild.getBoundingClientRect();
			var size2 = popup.contentDiv.getBoundingClientRect();
			  
			// if the first child of the popup content wants to be bigger than the available size
			if ( (size1.top - size1.bottom) != (size2.top - size2.bottom) ) {
			  popup.contentDiv.style.overflowY = 'scroll';
			}
		}
		this.lastCenter = theCenter;
	};
	
	
	this.buildPartMarker = function(part, color) {
		var markerLocation = null;
		var coords = [];
		var symbol = "*";
		var html = "";
		//We only want to map objects that have a specified lat/lon
		if ( part.latlon && part.latlon != "0|0") {
			coords = part.latlon.split(/\|/);

			markerLocation = new OpenLayers.LonLat( coords[1], coords[0] ).transform( this.map.displayProjection, this.map.projection );

			html = 	"<strong>(*)</strong> " + part.name;
			
			var marker = this.makeMarker(markerLocation, symbol, color, html);
			marker.html = html;
			this.subParts.push(marker);
		}
		return markerLocation;
	};
	
	this.buildLine = function(from, to, color) {
		this.subLines.push(this.dsAddLine(from, to, color));
	};
	
	this.getCenter = function(object) {
		if(object.latlon && object.latlon != "0|0") {
			var coords = object.latlon.split(/\|/);
			var center = new OpenLayers.LonLat( coords[1], coords[0] )
				.transform( this.map.displayProjection, this.map.projection );
		} else {
			return null;
		}
		
		return center;
	};
	
	//Expands a feature's parts
	this.expandParts = function(object, center, num, scope) {
		scope.modifyURL(num);
		scope.clearParts();
		
		if(object.latlon && object.latlon != "0|0") {
			var coords = object.latlon.split(/\|/);
			var origin = new OpenLayers.LonLat( coords[1], coords[0] )
				.transform( scope.map.displayProjection, scope.map.projection );
		} else {
			return;
		}
		
		if(object.parts != null) {
			var partLocation = null;
			for(i in object.parts) {
				partLocation = scope.buildPartMarker(object.parts[i], lightGreen);
				if(partLocation != null) {
					scope.buildLine(center, partLocation, lightGreen);
				}
			}
		}
		
		//Make a copy of the current set of features
		var remainder = scope.features.features.slice(0);
		//Remove it temoporarily
		scope.features.removeFeatures(scope.features.features);	
		scope.features.addFeatures(scope.subLines);
		scope.features.addFeatures(scope.subParts);	
		//Redraw it on top of the new parts and lines
		scope.features.addFeatures(remainder);
		scope.currentlyExpanded = num;
	};
	
	//Clears all parts and part lines from the map
	this.clearParts = function() {
		this.features.removeFeatures(this.subParts);
		this.features.removeFeatures(this.subLines);
		this.subParts = [];
		this.subLines = [];
	};
	
	this.expandObjectFromURL = function() {
		if(window.location.hash && this.lastHash != window.location.hash) {
			this.lastHash = window.location.hash;
			//Extract the slug from the hash portion of the URL
			var number = parseInt(this.lastHash.substring(this.lastHash.indexOf('#')+1, this.lastHash.length));
			var index = null;
			//Find the index at which the part number from the URL matches a part's number
			//Maybe there's built-in JS for this?
			for(i in this.partMarker) {
				if(this.partMarker[i].number == number) {
					index = i;
					break;
				}
			}
			if(index != null && typeof(this.partMarker[index].linkid) != 'undefined' && this.partMarker[index].linkid != 0 && this.currentlyExpanded != number) {
				this.doPost(this.partMarker[index].linkid, this.partMarker[index].geometry.getBounds().getCenterLonLat(), number, this);
			}
		}
	};
	
	this.modifyURL = function(num) {
		window.location.hash = num;
	};
	
	this.createMap( mapDiv );
	this.loadMap( config );
}

/**
 * Meters the world round
 */
var CIRCUM = 20037508.34 * 2; 

